#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Orifli - Simple Flickr Uploader for Gnome.

Features:
    1. Upload multiple photos.
    2. Ability to add titles, descriptions and tags to each photo.

'''

__author__ = 'Nikolay Blohin (nikolay@blohin.org)'
__version__ = '0.2.0'
__copyright__ = 'Copyright (c) 2010 Nikolay Blohin'
__license__ = 'GNU General Public License'

import os
import datetime
import threading
import gtk
import pango
import flickrapi


ABS_Path = os.path.realpath(os.path.dirname(__file__))
IMAGES_dir = os.path.join(ABS_Path, 'images')


class MainWindow(gtk.Window):

    def __init__(self):
        super(MainWindow, self).__init__()

        self.images = [] # all images for upload and it's data
        self.curent_folder = '' # folder for open dialog

        self.set_title('Orifli - Simple Flickr Uploader for Gnome')
        self.set_icon_from_file(os.path.join(IMAGES_dir, 'flickr-32x32.png'))
        self.set_default_size(500, 600)
        self.set_resizable(True)
        self.set_position(gtk.WIN_POS_CENTER)

        self.vbox = gtk.VBox()
        self.vbox_middle = gtk.VBox()
        hbox_1 = gtk.HBox(spacing=8)

        self.viewport = gtk.Viewport()
        self.viewport.add(self.vbox_middle)
        self.viewport.set_shadow_type(gtk.SHADOW_NONE)
        self.scroll = gtk.ScrolledWindow()
        self.scroll.add(self.viewport)
        self.scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

        toolbar = gtk.Toolbar()
        toolbar.set_style(gtk.TOOLBAR_BOTH)

        add_btn = gtk.ToolButton(gtk.STOCK_ADD)
        add_btn.connect('clicked', self.add_photo)
        upload_btn = gtk.ToolButton(gtk.STOCK_EXECUTE)
        upload_btn.connect('clicked', self.upload)
        exit_btn = gtk.ToolButton(gtk.STOCK_QUIT)
        exit_btn.connect('clicked', self.pr_exit)

        toolbar.insert(add_btn, 0)
        toolbar.insert(upload_btn, 1)
        toolbar.insert(exit_btn, -1)

        self.vbox.pack_start(toolbar, False, False)
        self.vbox.pack_start(gtk.HSeparator(), False, False)
        self.vbox.pack_start(self.scroll, True, True)

        self.add(self.vbox)
        self.set_border_width(10)

        self.connect('destroy', self.pr_exit)
        self.show_all()

    def redraw(self):
        ''' Refresh main window '''

        # clear main window
        all_for_del = self.vbox_middle.get_children()
        for a in all_for_del:
            a.destroy()

        # draw new content (all images)
        for image in self.images:
            img = gtk.Image()
            pixbuf = gtk.gdk.pixbuf_new_from_file(image['path'])

            # resize image
            old_width = pixbuf.get_width()
            old_height = pixbuf.get_height()
            if old_width >= old_height:
                koef = 150.0/old_width
            else:
                koef = 150.0/old_height
            tumb_w = int(old_width * koef)
            tumb_h = int(old_height * koef)
            scaled_buf = pixbuf.scale_simple(tumb_w, tumb_h, gtk.gdk.INTERP_BILINEAR)

            # draw image and buttons
            img.set_from_pixbuf(scaled_buf)
            img.set_tooltip_text(image['path'])
            vbox = gtk.VBox()
            button_privacy = gtk.Button('Set privacy')
            button_description = gtk.Button('Add description')
            button_delete = gtk.Button('Delete')

            button_privacy.set_size_request(120, 30)
            button_description.set_size_request(120, 30)
            button_delete.set_size_request(120, 30)

            button_privacy.connect('clicked', self.set_privacy)
            button_description.connect('clicked', self.set_description)
            button_delete.connect('clicked', self.del_photo)

            if image['uploaded']:
                label = gtk.Label('Successfully\nuploaded')
                button_clear = gtk.Button('Clear')
                button_clear.set_size_request(120, 30)
                button_clear.connect('clicked', self.del_photo)
                vbox.pack_start(label, False, False)
                vbox.pack_start(button_clear, False, False, padding=3)
            else:
                vbox.pack_start(button_privacy, False, False, padding=0)
                vbox.pack_start(button_description, False, False, padding=3)
                vbox.pack_start(button_delete, False, False, padding=0)

            textview = gtk.TextView()
            textview.set_editable(False)
            textview.set_left_margin(5)
            textview.set_right_margin(5)
            textview.set_wrap_mode(gtk.WRAP_WORD)
            my_buffer = textview.get_buffer()

            # register tag for decoration displayed messages
            table = my_buffer.get_tag_table()
            tag = gtk.TextTag('title')
            tag.set_property('foreground', '#999999')
            tag.set_property('size-points', 8)
            table.add(tag)

            my_iter = my_buffer.get_start_iter()
            my_buffer.insert_with_tags_by_name(my_iter, 'Privacy\n', 'title')
            my_buffer.insert(my_iter, image['privacy'])
            if image['title']:
                my_buffer.insert_with_tags_by_name(my_iter, '\n\nTitle\n', 'title')
                my_buffer.insert(my_iter, image['title'])
            if image['description']:
                my_buffer.insert_with_tags_by_name(my_iter, '\n\nDescription\n', 'title')
                my_buffer.insert(my_iter, image['description'])
            if image['tags']:
                my_buffer.insert_with_tags_by_name(my_iter, '\n\nTags\n', 'title')
                my_buffer.insert(my_iter, image['tags'])


            img_info_vbox = gtk.VBox()
            img_info_vbox.pack_start(textview, False, False)

            hbox = gtk.HBox(False)
            vbox2 = gtk.VBox()
            vbox2.pack_start(img, False, False)
            hbox.pack_start(vbox, False, False)
            hbox.pack_start(vbox2, False, False, padding=10)
            hbox.pack_start(img_info_vbox, True, True)

            hbox.my_id = image['id']

            self.vbox_middle.pack_start(hbox, False, False, padding=10)
            self.vbox_middle.pack_start(gtk.HSeparator(), False, False, padding=10)
        self.show_all()


    def add_photo(self, widget):
        ''' Add new photo for upload '''
        sel_photo = gtk.FileChooserDialog(title='Select photo',
                                          action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                          buttons=(gtk.STOCK_CANCEL,
                                                   gtk.RESPONSE_CANCEL,
                                                   gtk.STOCK_OPEN,
                                                   gtk.RESPONSE_OK))
        sel_photo.set_select_multiple(True)
        sel_photo.set_current_folder(self.curent_folder)

        # Create and add the 'images' filter
        img_filter = gtk.FileFilter()
        img_filter.set_name('Images')
        img_filter.add_mime_type('image/png')
        img_filter.add_mime_type('image/jpeg')
        img_filter.add_mime_type('image/gif')
        img_filter.add_mime_type('image/bmp')
        img_filter.add_pattern('*.png')
        img_filter.add_pattern('*.jpg')
        img_filter.add_pattern('*.gif')
        img_filter.add_pattern('*.bmp')
        sel_photo.add_filter(img_filter)

        # Create and add the 'all files' filter
        all_filter = gtk.FileFilter()
        all_filter.set_name('All files')
        all_filter.add_pattern('*')
        sel_photo.add_filter(all_filter)

        # run select file dialog
        response = sel_photo.run()
        path_to_imgs = sel_photo.get_filenames()
        self.curent_folder = sel_photo.get_current_folder()
        sel_photo.destroy()
        if response != gtk.RESPONSE_OK:
            return

        for path_to_img in path_to_imgs:
            try:
                pixbuf = gtk.gdk.pixbuf_new_from_file(path_to_img)
            except:
                continue

            # add image to main images list
            image = {'path':path_to_img,
                     'privacy':'public',
                     'id':str(datetime.datetime.now()),
                     'title':'',
                     'description':'',
                     'tags':'',
                     'uploaded':False}
            self.images.append(image)
        self.redraw()

    def set_privacy(self, widget):
        ''' Set privacy police for photo '''
        id_for_change = widget.get_parent().get_parent().my_id

        counter = 0
        for i in self.images:
            if i['id'] == id_for_change:
                curent_img = counter
                break
            counter += 1

        dialog = gtk.Dialog('Set privacy',
                           None,
                           gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                           (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                            gtk.STOCK_OK, gtk.RESPONSE_OK))

        button1 = gtk.RadioButton(None, 'Private (only you see them)')
        button2 = gtk.RadioButton(button1, 'Public (anyone can see them)')

        checkbox1 = gtk.CheckButton('Visible to Friends')
        checkbox2 = gtk.CheckButton('Visible to Family')

        if self.images[curent_img]['privacy'] == 'private (friends and family)':
            button1.set_active(True)
            checkbox1.set_active(True)
            checkbox2.set_active(True)
        elif self.images[curent_img]['privacy'] == 'private (friends)':
            button1.set_active(True)
            checkbox1.set_active(True)
        elif self.images[curent_img]['privacy'] == 'private (family)':
            button1.set_active(True)
            checkbox2.set_active(True)
        elif self.images[curent_img]['privacy'] == 'private':
            button1.set_active(True)
        else:
            button2.set_active(True)

        fix = gtk.Fixed()
        fix.put(button2, 10, 10)
        fix.put(button1, 10, 40)
        fix.put(checkbox1, 30, 65)
        fix.put(checkbox2, 30, 90)
        dialog.vbox.pack_start(fix)
        fix.show_all()

        response = dialog.run()
        dialog.destroy()
        if response != gtk.RESPONSE_OK:
            return

        if button2.get_active():
            self.images[curent_img]['privacy'] = 'public'
        else:
            if checkbox1.get_active() and checkbox2.get_active():
                self.images[curent_img]['privacy'] = 'private (friends and family)'
            elif checkbox1.get_active():
                self.images[curent_img]['privacy'] = 'private (friends)'
            elif checkbox2.get_active():
                self.images[curent_img]['privacy'] = 'private (family)'
            else:
                self.images[curent_img]['privacy'] = 'private'

        self.redraw()



    def set_description(self, widget):
        ''' Add describe the photo '''
        id_for_change = widget.get_parent().get_parent().my_id

        # define curent image in global images list
        counter = 0
        for i in self.images:
            if i['id'] == id_for_change:
                curent_img = counter
                break
            counter += 1

        dialog = gtk.Dialog('Describe the photo',
                           None,
                           gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                           (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                            gtk.STOCK_OK, gtk.RESPONSE_OK))
        dialog.set_size_request(300, 300)
        dialog.set_border_width(5)

        label_title = gtk.Label('<small>Title</small>')
        label_description = gtk.Label('<small>Description</small>')
        label_tags = gtk.Label('<small>Tags</small>')

        label_title.set_use_markup(True)
        label_description.set_use_markup(True)
        label_tags.set_use_markup(True)

        label_title.set_alignment(0, 0)
        label_description.set_alignment(0, 0)
        label_tags.set_alignment(0, 0)

        entry_title = gtk.Entry()
        entry_tags = gtk.Entry()

        entry_title.set_text(self.images[curent_img]['title'])
        entry_tags.set_text(self.images[curent_img]['tags'])

        textview = gtk.TextView()
        scroll = gtk.ScrolledWindow()
        scroll.add(textview)
        scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        textview.set_wrap_mode(gtk.WRAP_WORD)
        my_buffer = textview.get_buffer()

        my_buffer.set_text(self.images[curent_img]['description'])

        content = gtk.VBox(False)
        content.pack_start(label_title, False, False)
        content.pack_start(entry_title, False, False)
        content.pack_start(gtk.HSeparator(), False, False, padding=10)
        content.pack_start(label_description, False, False)
        content.pack_start(scroll, True, True)
        content.pack_start(gtk.HSeparator(), False, False, padding=10)
        content.pack_start(label_tags, False, False)
        content.pack_start(entry_tags, False, False)
        dialog.vbox.pack_start(content)
        content.show_all()

        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            self.images[curent_img]['title'] = entry_title.get_text()
            self.images[curent_img]['tags'] = entry_tags.get_text()
            self.images[curent_img]['description'] = my_buffer.get_text(my_buffer.get_start_iter(), my_buffer.get_end_iter())

        dialog.destroy()
        self.redraw()



    def del_photo(self, widget):
        ''' Delete photo from window '''
        id_for_del = widget.get_parent().get_parent().my_id
        self.images = filter(lambda x: x['id'] != id_for_del, self.images)
        self.redraw()

    def upload(self, widget):
        ''' Upoad selected photos to Flickr '''

        def load_in_thread(self, images):

            def func_progress(progress, done):
                gtk.gdk.threads_enter()
                if done:
                    progressbar.set_fraction(1.0)
                else:
                    progressbar.set_fraction(progress/100)
                gtk.gdk.threads_leave()

            for image in images:
                if image['uploaded']:
                    continue
                # show progressbar
                gtk.gdk.threads_enter()
                progressbar = gtk.ProgressBar()
                progressbar.set_fraction(0.0)
                progressbar.set_text(image['path'])
                self.vbox.pack_end(progressbar, False, False)
                progressbar.show()
                gtk.gdk.threads_leave()

                if image['privacy'] == 'public':
                    my_is_public = 1
                    my_is_family = 0
                    my_is_friend = 0
                elif image['privacy'] == 'private (friends and family)':
                    my_is_public = 0
                    my_is_family = 1
                    my_is_friend = 1
                elif image['privacy'] == 'private (friends)':
                    my_is_public = 0
                    my_is_family = 0
                    my_is_friend = 1
                elif image['privacy'] == 'private (family)':
                    my_is_public = 0
                    my_is_family = 1
                    my_is_friend = 0
                elif image['privacy'] == 'private':
                    my_is_public = 0
                    my_is_family = 0
                    my_is_friend = 0

                try:
                    rezult = flickr.upload(filename=image['path'],
                                           title=image['title'],
                                           description=image['description'],
                                           tags=image['tags'],
                                           is_public=my_is_public,
                                           is_family=my_is_family,
                                           is_friend=my_is_friend,
                                           callback=func_progress)

                    image['uploaded'] = True
                except:
                    pass

                if image['uploaded']:
                    gtk.gdk.threads_enter()
                    # deactivate all buttons and add upload message
                    all_vbox = self.vbox_middle.get_children()
                    for ph in all_vbox:
                        if hasattr(ph, 'my_id') and ph.my_id == image['id']:
                            box = ph.get_children()[0]
                            all_buttons = box.get_children()
                            for b in all_buttons:
                                b.destroy()
                            label = gtk.Label('Successfully\nuploaded')
                            button_clear = gtk.Button('Clear')
                            button_clear.set_size_request(120, 30)
                            button_clear.connect('clicked', self.del_photo)
                            box.pack_start(label, False, False)
                            box.pack_start(button_clear, False, False)
                            box.show_all()
                    gtk.gdk.threads_leave()

                gtk.gdk.threads_enter()
                progressbar.destroy()
                gtk.gdk.threads_leave()


        # connect to Flickr
        if self.images:
            api_key = 'b255afa642484ec546fecb785483dc2d'
            api_secret = 'a0d4544051191e5d'
            flickr = flickrapi.FlickrAPI(api_key, api_secret)
            (token, frob) = flickr.get_token_part_one(perms='write')
            if not token:
                md = gtk.MessageDialog(self,
                    gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_INFO,
                    gtk.BUTTONS_OK, 'Press OK after you authorized this program')
                md.run()
                md.destroy()
            flickr.get_token_part_two((token, frob))

            mythread = threading.Thread(target=load_in_thread, args=(self, self.images))
            mythread.start()


    def pr_exit(self, widget):
        ''' Exit from program '''
        gtk.main_quit()


gtk.gdk.threads_init()
MainWindow()
gtk.main()
